/*
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: hxf <hewenfei@kylinos.cn>
 *
 */

#ifndef UKUI_QUICK_ISLAND_VIEW_H
#define UKUI_QUICK_ISLAND_VIEW_H

#include <QQuickWindow>
#include <QQmlContext>
#include <QUrl>

#include "widget-container-item.h"
#include "island.h"

namespace UkuiQuick {

class IslandViewPrivate;

/**
 * 1.用于加载ContainerItem的窗口
 *
 * 封装加载逻辑，简化加载流程
 */
class IslandView : public QQuickWindow
{
    Q_OBJECT
public:
    IslandView() = delete;
    explicit IslandView(const QString &id, const QString &app);
    ~IslandView() override;

    /**
     * 在搜索路径中自动寻找该插件并加载
     * 前提：需要将Container制作为一个Widget包
     * TODO: 实现它!
     */
    bool loadMainView(const QString &id);

    /**
     * 设置一个Container作为根View
     * @param view
     */
    void setMainView(WidgetContainer *view);
    WidgetContainer *mainView() const;

    /**
     * 加载Container的ui
     * 不同的Container往往UI也不相同，需要按情况编写并加载
     * @param source ui文件的地址
     * TODO: 使用loader自动进行加载
     */
    void setSource(const QUrl &source);

    // mainView UI组件的上下文
    QQmlContext *rootContext() const;
    WidgetContainerItem *rootItem() const;

    Config *config() const;
    Island *island() const;

private:
    IslandViewPrivate *d {nullptr};
};

} // UkuiQuick

#endif //UKUI_QUICK_ISLAND_VIEW_H
