/*
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: hxf <hewenfei@kylinos.cn>
 *
 */

#ifndef UKUI_QUICK_WIDGET_ITEM_H
#define UKUI_QUICK_WIDGET_ITEM_H

#include <QObject>
#include <QQuickItem>

namespace UkuiQuick {

class Widget;
class WidgetItemPrivate;

/**
 * Widget的图形组件，基于QQuickItem(QML)
 * 包含图形ui全部信息
 * 提供上下文属性widgetItem用于访问Widget实例
 */
class WidgetItem : public QQuickItem
{
    Q_OBJECT
public:
    // 为Widget创建图形组件
    static WidgetItem *itemForWidget(Widget *widget);
    static void registerTypes();

    ~WidgetItem() override;
    Widget *widget() const;

protected:
    void classBegin() override;

public:
    explicit WidgetItem(QQuickItem *parent = nullptr);

private:
    WidgetItemPrivate *d {nullptr};
};

} // UkuiQuick

#endif //UKUI_QUICK_WIDGET_ITEM_H
